[Python-checkins] cpython (3.4): asyncio: ensure_future() now understands awaitables

yury.selivanov python-checkins at python.org
Fri Oct 2 15:10:13 EDT 2015


https://hg.python.org/cpython/rev/b40a61e79893
changeset:   98486:b40a61e79893
branch:      3.4
parent:      98481:3940f480ea16
user:        Yury Selivanov <yselivanov at sprymix.com>
date:        Fri Oct 02 15:00:19 2015 -0400
summary:
  asyncio: ensure_future() now understands awaitables

files:
  Lib/asyncio/tasks.py                |  16 +++++++++++++-
  Lib/test/test_asyncio/test_tasks.py |  18 +++++++++++++++++
  2 files changed, 32 insertions(+), 2 deletions(-)


diff --git a/Lib/test/test_asyncio/test_tasks.py b/Lib/test/test_asyncio/test_tasks.py
--- a/Lib/test/test_asyncio/test_tasks.py
+++ b/Lib/test/test_asyncio/test_tasks.py
@@ -153,6 +153,24 @@
         t = asyncio.ensure_future(t_orig, loop=self.loop)
         self.assertIs(t, t_orig)
 
+    @unittest.skipUnless(PY35, 'need python 3.5 or later')
+    def test_ensure_future_awaitable(self):
+        class Aw:
+            def __init__(self, coro):
+                self.coro = coro
+            def __await__(self):
+                return (yield from self.coro)
+
+        @asyncio.coroutine
+        def coro():
+            return 'ok'
+
+        loop = asyncio.new_event_loop()
+        self.set_event_loop(loop)
+        fut = asyncio.ensure_future(Aw(coro()), loop=loop)
+        loop.run_until_complete(fut)
+        assert fut.result() == 'ok'
+
     def test_ensure_future_neither(self):
         with self.assertRaises(TypeError):
             asyncio.ensure_future('ok')

diff --git a/Lib/asyncio/tasks.py b/Lib/asyncio/tasks.py
--- a/Lib/asyncio/tasks.py
+++ b/Lib/asyncio/tasks.py
@@ -512,7 +512,7 @@
 
 
 def ensure_future(coro_or_future, *, loop=None):
     """Wrap a coroutine in a future.
 
     If the argument is a Future, it is returned directly.
     """
@@ -527,8 +527,20 @@
         if task._source_traceback:
             del task._source_traceback[-1]
         return task
+    elif compat.PY35 and inspect.isawaitable(coro_or_future):
+        return ensure_future(_wrap_awaitable(coro_or_future), loop=loop)
     else:
-        raise TypeError('A Future or coroutine is required')
+        raise TypeError('A Future, a coroutine or an awaitable is required')
+
+
+ at coroutine
+def _wrap_awaitable(awaitable):
+    """Helper for asyncio.ensure_future().
+
+    Wraps awaitable (an object with __await__) into a coroutine
+    that will later be wrapped in a Task by ensure_future().
+    """
+    return (yield from awaitable.__await__())
 
 
 class _GatheringFuture(futures.Future):
@@ -512,7 +512,7 @@
 
 def ensure_future(coro_or_future, *, loop=None):
-    """Wrap a coroutine in a future.
+    """Wrap a coroutine or an awaitable in a future.
 
     If the argument is a Future, it is returned directly.
-- 
Repository URL: https://hg.python.org/cpython


More information about the Python-checkins mailing list